/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Robert Sayre <sayrer@gmail.com> (original author)
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "domstubs.idl"

[scriptable, uuid(D894B5D4-44F3-422A-A220-7763C12D4A94)]
interface nsIDOMNSElement : nsISupports
{
  /*
   * Retrieve elements matching all classes listed in a
   * space-separated string.
   *
   * See <http://whatwg.org/specs/web-apps/current-work/>
   */
  nsIDOMNodeList getElementsByClassName(in DOMString classes);

  /*
   * Retrieve a list of rectangles, one for each CSS border-box associated with
   * the element. The coordinates are in CSS pixels, and relative to
   * the top-left of the document's viewport, unless the document
   * has an SVG foreignobject ancestor, in which case the coordinates are
   * relative to the top-left of the content box of the nearest SVG foreignobject
   * ancestor. The coordinates are calculated as if every scrollable element
   * is scrolled to its default position.
   *
   * Note: the boxes of overflowing children do not affect these rectangles.
   * Note: some elements have empty CSS boxes. Those return empty rectangles,
   * but the coordinates may still be meaningful.
   * Note: some elements have no CSS boxes (including display:none elements,
   * HTML AREA elements, and SVG elements that do not render). Those return
   * an empty list.
   */
  nsIDOMClientRectList getClientRects();

  /**
   * Returns the union of all rectangles in the getClientRects() list. Empty
   * rectangles are ignored, except that if all rectangles are empty,
   * we return an empty rectangle positioned at the top-left of the first
   * rectangle in getClientRects().
   */
  nsIDOMClientRect getBoundingClientRect();

  /**
   * The vertical scroll position of the element, or 0 if the element is not
   * scrollable. This property may be assigned a value to change the
   * vertical scroll position.
   */
  attribute long             scrollTop;

  /**
   * The horizontal scroll position of the element, or 0 if the element is not
   * scrollable. This property may be assigned a value to change the
   * horizontal scroll position.
   */
  attribute long             scrollLeft;

  /**
   * The height of the scrollable area of the element. If the element is not
   * scrollable, scrollHeight is equivalent to the offsetHeight.
   */
  readonly attribute long             scrollHeight;

  /**
   * The width of the scrollable area of the element. If the element is not
   * scrollable, scrollWidth is equivalent to the offsetWidth.
   */
  readonly attribute long             scrollWidth;

  /**
   * The height in CSS pixels of the element's top border.
   */
  readonly attribute long             clientTop;

  /**
   * The width in CSS pixels of the element's left border and scrollbar
   * if it is present on the left side.
   */
  readonly attribute long             clientLeft;

  /**
   * The width in CSS pixels of the element's padding box. If the element is
   * scrollable, the scroll bars are included inside this height.
   */
  readonly attribute long             clientHeight;

  /**
   * The height in CSS pixels of the element's padding box. If the element is
   * scrollable, the scroll bars are included inside this width.
   */
  readonly attribute long             clientWidth;

  /**
   * Similar as the attributes on nsIDOMNode, but navigates just elements
   * rather than all nodes.
   *
   * Defined by the ElementTraversal spec.
   */
  readonly attribute nsIDOMElement firstElementChild;
  readonly attribute nsIDOMElement lastElementChild;
  readonly attribute nsIDOMElement previousElementSibling;
  readonly attribute nsIDOMElement nextElementSibling;

  /**
   * Returns the number of child nodes that are nsIDOMElements.
   *
   * Defined by the ElementTraversal spec.
   */
  readonly attribute unsigned long childElementCount;
  
  /**
   * Returns a live nsIDOMNodeList of the current child elements.
   */
  readonly attribute nsIDOMNodeList children;

  /**
   * Returns a DOMTokenList object reflecting the class attribute.
   */
  readonly attribute nsIDOMDOMTokenList classList;

  /**
   * Set this during a mousedown event to grab and retarget all mouse events
   * to this element until the mouse button is released or releaseCapture is
   * called. If retargetToElement is true, then all events are targetted at
   * this element. If false, events can also fire at descendants of this
   * element.
   * 
   */
  void setCapture([optional] in boolean retargetToElement);

  /**
   * If this element has captured the mouse, release the capture. If another
   * element has captured the mouse, this method has no effect.
   */
  void releaseCapture();

  /**
   * Returns whether this element would be selected by the given selector
   * string.
   *
   * See <http://dev.w3.org/2006/webapi/selectors-api2/>
   */
  boolean mozMatchesSelector([Null(Stringify)] in DOMString selector);
};
